#!/bin/sh
# Reference: 80-y0615-3_a_qca8081_hardware_register_description.pdf

########################################################
# Definition
########################################################
BOARD_CFG="/etc/factory.d/board.cfg"

if [ -e $BOARD_CFG ]; then
	. $BOARD_CFG
else
	echo "$BOARD_CFG is not exist!";
	exit 1;
fi

NAPA_PHY_LED_CFG="/etc/factory.d/napa_phy_led.cfg"

if [ -e $NAPA_PHY_LED_CFG ]; then
	. $NAPA_PHY_LED_CFG
else
	echo "$NAPA_PHY_LED_CFG is not exist!";
	exit 1;
fi

########################################################
# Debug message
########################################################
DEBUG=2;

DBG() {
	[ $DEBUG -gt 0 ] && echo "=> $@" > /dev/console
}
dbg() {
	[ $DEBUG -gt 1 ] &&	echo "$@" > /dev/console
}


########################################################
# sub-Functions
########################################################
#-----------------------------------------------------
# usage
#-----------------------------------------------------
usage() {
	echo "Usage:"
	echo "napa_phy_led.sh [phy_addr] [led0_state] [led1_state] [led2_state]"
	echo "e.g.: napa_phy_led.sh 24 1 0 1"
}

#-----------------------------------------------------
# mmg_reg_write
# $1=MMD_NUM
# $2=PHY_ADDR
# $3=MMD_REG_ADDR
# $4=MMD_REG_VALUE
#-----------------------------------------------------
mmd_reg_write()
{
	MMD_ACCESS_CONTROL_REG=0xD;
	MMD_ACCESS_DATA_REG=0xE;
	MMD_NUM=$1;
	PHY_ADDR=$2;
	MMD_REG_ADDR=$3;
	MMD_REG_VALUE=$4;
	RM_MMD_REG=`printf "0x40%02x\n" $MMD_NUM`;

	echo "MMD$1 write [$3] [$4]";

	ssdk_sh debug phy set $PHY_ADDR $MMD_ACCESS_CONTROL_REG $MMD_NUM > /dev/null;
	ssdk_sh debug phy set $PHY_ADDR $MMD_ACCESS_DATA_REG $MMD_REG_ADDR > /dev/null;
	ssdk_sh debug phy set $PHY_ADDR $MMD_ACCESS_CONTROL_REG $RM_MMD_REG > /dev/null;
	ssdk_sh debug phy set $PHY_ADDR $MMD_ACCESS_DATA_REG $MMD_REG_VALUE
}

#-----------------------------------------------------
# napa_phy_led_set
# $1=phy_addr
# $2=led0_state
# $3=led1_state
# $4=led2_state
#-----------------------------------------------------
napa_phy_led_set() {

    local MMD7_LED0_CTRL_ADDR="0x8078"
    local MMD7_LED1_CTRL_ADDR="0x8074"
    local MMD7_LED2_CTRL_ADDR="0x8076"
    local phy_addr=$1;
    local led0_state=$2;
    local led1_state=$3;
    local led2_state=$4;

	if [ $led0_state == "0" ]; then
		mmd_reg_write 7 $phy_addr $MMD7_LED0_CTRL_ADDR 0x0
	else
		mmd_reg_write 7 $phy_addr $MMD7_LED0_CTRL_ADDR 0x8070
	fi

	if [ $led1_state == "0" ]; then
		mmd_reg_write 7 $phy_addr $MMD7_LED1_CTRL_ADDR 0x0
	else
		mmd_reg_write 7 $phy_addr $MMD7_LED1_CTRL_ADDR 0x8070
	fi

	if [ $led2_state == "0" ]; then
		mmd_reg_write 7 $phy_addr $MMD7_LED2_CTRL_ADDR 0x0
	else
		mmd_reg_write 7 $phy_addr $MMD7_LED2_CTRL_ADDR 0x8070
	fi
}


########################################################
# MAIN
########################################################
DBG "CPU: $CPU";
DBG "ethernet: $ETH";
DBG "#=$#; 1=$1; 2=$2, 3=$3, 4=$4"

printf "\n\\033[33m*** Please ensure the phy link is up before the test (10/100/1000/2500M all are OK)***\\033[0m\\n\n"

case $CPU in
	"IPQ8070" | "IPQ8072" | "IPQ8074")
		case $ETH in
			"QCA8081" | "AQR112C_QCA8081")
                if [ $# == "4" ]; then
                    napa_phy_led_set $1 $2 $3 $4;
                else
                    usage;
                    exit 0;
                fi
				;;
			*)
				echo "unknow ethernet type!";
				exit 1;
				;;
		esac
		;;
	*)
		echo "unknown CPU type!";
		exit 1;
		;;
esac
