#!/bin/sh

## global parameters ##
# WDSB role auto-detection
PROC_SN_ROLE="/proc/nawds_role"
currentrole="0"
[ -e "$PROC_SN_ROLE" ] && currentrole=$(cat $PROC_SN_ROLE)
ping_retry_limit=3
debugMsgEnable=0

ifname=$1

## sub function ##
set_wds_node()
{
    [ "$debugMsgEnable" == "1" ] && echo -e "\033[33m set_wds_node \033[0m" > /dev/console
    if [ "$currentrole" == "2" ]; then
        exit 0
    else
        echo 2 > $PROC_SN_ROLE
        cfg80211tool "$ifname" update_ie 1
    fi
}

set_wds_root()
{
    [ "$debugMsgEnable" == "1" ] && echo -e "\033[33m set_wds_root \033[0m" > /dev/console
    if [ "$currentrole" == "1" ]; then
        exit 0
    else
        echo 1 > $PROC_SN_ROLE
        cfg80211tool "$ifname" update_ie 1
    fi
}

## main function ##

#[ ! -e "/tmp/booting_is_done" ] && exit

[ -e "/var/run/luci-reload-status" ] && exit #exit if luci reload init scripts

[ -e "/tmp/mesh_syncing" -o -e "/tmp/mode_switching" -o -e "/tmp/mode_switching_network_reload" ] && exit  #exit if syncing or reloading, it may makes something error

wifi0_opmode=$(uci get wireless.wifi0.opmode)
wifi1_opmode=$(uci get wireless.wifi1.opmode)

if [ "$wifi0_opmode" == "wds_ap" -o "$wifi1_opmode" == "wds_ap" ]; then
    if [ "$debugMsgEnable" == "1" ]; then
        echo === WDS-AP is enable, pass WDS-Bridge checking === >> /dev/console
    fi
    set_wds_root
    exit 0

elif [ "$wifi0_opmode" != "wds_bridge" -a "$wifi1_opmode" != "wds_bridge" ]; then
    if [ "$debugMsgEnable" == "1" ]; then
        echo === WDS-Bridge is disabled, clean checking function in crontab === >> /dev/console
    fi
    sed -i '/chkEth2WDSB/d' /etc/crontabs/root
    crontab /etc/crontabs/root
    exit 0
fi

gatewayIP=$(route -n| grep 'UG[ \t]'|awk '{ print $2 }')

if [ -z "$gatewayIP" ] || [ "$gatewayIP" == "0.0.0.0" ]; then
    if [ "$debugMsgEnable" == "1" ]; then
        echo === error! device does not have gateway ip ==== >> /dev/console
    fi
    set_wds_node
    exit 0
fi

devIP=$(ifconfig br-lan | awk '/inet addr/{print substr($2,6)}')

if [ -z "$devIP" ]; then
    if [ "$debugMsgEnable" == "1" ]; then
        echo ===error! device does not have ip address === >> /dev/console
    fi
    set_wds_node
    exit 0
fi

tmp_if=$(uci get network.lan.ifname)
ethInterface=$(echo $tmp_if | grep ^eth | awk {'print $1'})
[ "$debugMsgEnable" == "1" ] && echo "ethInterface:[$ethInterface]" > /dev/console

if [ -z $ethInterface ]; then
    if [ "$debugMsgEnable" == "1" ]; then
        echo === error! device does not have ethernet interface === >> /dev/console
    fi
    set_wds_node
    exit 0
else
    chkEth=$(cat /sys/class/net/"$ethInterface"/carrier)
    if [ "$chkEth" != "1" ]; then
        if [ "$debugMsgEnable" == "1" ]; then
            echo === ethernet not plugged in, set mesh ap directly === >> /dev/console
        fi
        set_wds_node
        exit 0
    fi
fi

gatewayMac=$(cat /proc/net/arp | grep "$gatewayIP " | awk '{print $4}')

if [ -z "$gatewayMac" ]; then		#arping the gateway mac to refresh the arp table
    arping -I br-lan $gatewayIP -c 1	
    gatewayMac=$(cat /proc/net/arp | grep "$gatewayIP " | awk '{print $4}')
fi

for i in `seq 1 $ping_retry_limit`
do
    if [ -z "$gatewayMac" ]; then
        pingAns=$(interface_ping -I $ethInterface -S $devIP -T $gatewayIP -d 1 | grep "bytes received" | awk '{print $1}')
    else
        pingAns=$(interface_ping -I $ethInterface -S $devIP -T $gatewayIP -d 1 -M $gatewayMac | grep "bytes received" | awk '{print $1}')
    fi

    if [ "$pingAns" == "$gatewayIP" ]; then 
        if [ "$debugMsgEnable" == "1" ]; then 
            echo pingAns=$pingAns gatewayIP=$gatewayIP >> /dev/console
        fi
        set_wds_root
        exit 0
    else
        if [ "$debugMsgEnable" == "1" ]; then
            echo ping failed $i times >> /dev/console
        fi
    fi
done

set_wds_node

## main function end ##
