#!/bin/sh

. /lib/functions.sh
. /usr/share/libubox/jshn.sh

debug=0
debug_file=/tmp/dbg_crashdump

dbg()
{
  msg=$1
  [ "$debug" = "1" ] && echo "[crashdump] $msg" >> $debug_file
}

prepare_files()
{
    local cfg=crashdump
    local path
    local timestamp=$(date "+%Y%m%dT%H%M%S")
    config_load crashdump
    config_get path "$cfg" path "/usr/share/crashdump/"

    if [ -n "$(ls -A /usr/share/crashdump)" -a ! -n "$(ls -A /tmp/crashdump_20*.tar)" ]; then
            tar cf /tmp/crashdump_$timestamp.tar $path > /dev/null 2>&1
            echo "crashdump_$timestamp.tar"
    elif [ -n "$(ls -A /tmp/crashdump_20*.tar)" ]; then
            echo $(ls -A /tmp/crashdump_20*.tar | awk -F'/' '{print $3}')
    else
            echo ""
    fi
}

get_url()
{
    local fname=$1
    local retval
    local cfg=crashdump
    local ezmurl
    local address
    local scheme
    local port
    local crashdump_path
    config_load ezmcloud
    config_get address tortoise address "device.production.engenius.ai"
    config_get scheme tortoise scheme "https"
    config_get port toroise port "443"
    # crashdump_path must be /kdumps
    config_get crashdump_path tortoise crashdump_path "/kdumps"

    if [ "${address#*/}" = "$address" ]
    then
        ezmurl="$scheme://$address:$port/${crashdump_path#/}"
    else
        ezmurl="$scheme://${address%%/*}:$port/${address#*/}/${crashdump_path#/}"
    fi

    dbg "[get_url] ezmurl: $ezmurl"
    # curl get, save response json format to res
    res=$(curl -f -H "Content-Type: application/json" --connect-timeout 5 -m 10 -d "{\"filename\":\"$fname\"}" --cert /etc/ezmcloud/client.crt --key /etc/ezmcloud/client.key -k $ezmurl)
    retval=$?
    [ "$retval" = 0 ] || (echo "";return $retval)

    json_init
    json_load "$res"
    #json_dump 
    json_get_var var upload_url
    if [ -n "$var" ]; then
        echo $var
    else
        echo ""
    fi
}

upload_config()
{
    local url=$1
    local file=/tmp/$2
    local retval
    $(curl -f -X PUT --data-binary "@$file" --header "Content-Type: application/octet-stream" --cert /etc/ezmcloud/client.crt --key /etc/ezmcloud/client.key -k $url)
    retval=$?
    return $retval
}

run_upload()
{
    local retval
    local fname
    local url
    local cfg=crashdump
    local path
    local ldebug
    config_load crashdump
    config_get path "$cfg" path "/usr/share/crashdump/"
    config_get ldebug "$cfg" debug "0"
    debug=$ldebug
    fname=$(prepare_files)
    dbg "[run_upload] fname: $fname"
    [ -z "$fname" ] && return 0

    url=$(get_url $fname)
    dbg "[run_upload] get upload url: $url"
    [ -z "$url" ] && return 0

    upload_config $url $fname
    retval=$?
    dbg "[run_upload] upload_config result:$retval"
    [ "$retval" = "0" ] && rm -f /usr/share/crashdump/* && rm -f /tmp/$fname && sed -i '/crashdump_upload.sh/d' /etc/crontabs/root
}


run_upload
